@@ -45,6 +45,12 @@
 #'  smaller than \code{sig.level}.  The BIC for a given degree \code{m} is calculated as in
 #'  Schwarz (1978) where the dimension of the model is \eqn{d = \#\{i: \hat p_i\ge\epsilon,
 #'   i = 0, \ldots, m\} - 1} and a default \eqn{\epsilon} is chosen as \code{.Machine$double.eps}.
+#'
+#'  If data show a clearly multimodal distribution by plotting the histogram for example, 
+#'  the model degree is usually large. The range \code{M} should be large enough to cover the 
+#'  optimal degree and the computation is time-consuming. In this case the iterative method 
+#'  of moment with an initial selected by a method of mode which is implemented
+#'  by \code{\link{optimable}} can be used to reduce the computation time.
 #' @return  A list with components
 #' \itemize{
 #'   \item \code{m} the given or a selected degree by method of change-point
@@ -72,9 +78,11 @@
 #' }
 #' @note Since the Bernstein polynomial model of degree \eqn{m} is nested in the model of
 #' degree \eqn{m+1}, the maximum likelihood is increasing in \eqn{m}. The change-point method
-#' is used to choose an optimal degree \eqn{m}.
+#' is used to choose an optimal degree \eqn{m}. The degree can also be chosen by a method of
+#' moment and a method of mode which are implemented by function \code{optimal()}.
 #' @author Zhong Guan <zguan@iusb.edu>
 #' @references Guan, Z. (2016) Efficient and robust density estimation using Bernstein type polynomials. \emph{Journal of Nonparametric Statistics}, 28(2):250-271.
+#' @seealso \code{\link{optimable}}
 #' @examples
 #' \donttest{
 #' # Vaal Rive Flow Data
@@ -155,9 +163,9 @@ mable<-function(x, M, interval = c(0,1), IC = c("none", "aic", "hqic", "all"),
     }
     xbar<-mean(x); s2<-var(x); 
     m0<-max(0,ceiling(xbar*(1-xbar)/s2-3)-2)
-    if(M[1]<m0){
-      message("Replace M[1]=",M[1]," by the recommended ", m0,".")
-      M[1]=m0; M[2]=max(m0,M[2])}
+    #if(M[1]<m0){
+    #  message("Replace M[1]=",M[1]," by the recommended ", m0,".")
+    #  M[1]=m0; M[2]=max(m0,M[2])}
     k<-M[2]-M[1]
     if(k>0 && k<=3){
       message("Too few candidate model degrees. Add 5 more.")
@@ -339,9 +347,9 @@ mable.group<-function(x, breaks, M, interval=c(0, 1), IC=c("none", "aic", "hqic"
     x1<-rep.int((t[-N]+t[-1])/2,times=x)
     xbar<-mean(x1); s2<-var(x1); 
     m0<-max(0,ceiling(xbar*(1-xbar)/s2-3)-2)
-    if(M[1]<m0){
-      message("Replace M[1]=",M[1]," by the recommended ", m0,".")
-      M[1]=m0; M[2]=max(m0,M[2])}
+    #if(M[1]<m0){
+    #  message("Replace M[1]=",M[1]," by the recommended ", m0,".")
+    #  M[1]=m0; M[2]=max(m0,M[2])}
     if(any(t<0) | any(t>1)) stop("'breaks' must be in 'interval'")
     convergence<-0
     del<-controls$eps
@@ -410,6 +418,334 @@ mable.group<-function(x, breaks, M, interval=c(0, 1), IC=c("none", "aic", "hqic"
     class(ans)<-"mable"
     return(ans)
 }
+###########################################################
+#'  check if sequence contains repeated subvector
+#' @keywords internal
+#' @noRd
+rep.check <- function(x) {
+  i <- 0
+  while (TRUE) {
+    i <- i + 1
+    if(sum((diff(x, lag = i) == 0) - 1)==0)
+      break 
+  }
+  list(Period = i, Subseq = x[1:i], Reps = length(x)/i) 
+}
+
+###########################################################
+#' Upper confidence limit for density at modes
+#' @description Calculate a rough upper confidence limit for a density f at modes
+#'  using kernel density estimate with Gaussian kernel for the purpose 
+#' of choosing Bernstein polynomial model degree
+# 
+#' @param x a vector of sample data
+#' @param z a vector of mode(s), default is NULL. Then \code{z} is a vector of  
+#'     \code{nz} modes returned by \code{multimode::locmodes()}.
+#' @param nz known number of mode(s), must be given if \code{z=NULL}.
+#' @param alpha used to specify the confidence level
+#' @param interval support/truncation interval
+#' @param del small positive number used to calculate density histogram at 0/1 
+#' @keywords internal
+#' @author Zhong Guan <zguan@iusb.edu>
+#' @importFrom stats qnorm 
+# @importFrom ks kde 
+# @importFrom multimode locmodes 
+#' @noRd
+ucl.fz<-function(x, z=NULL, nz=length(z), alpha=0.05, interval=c(0,1), del=1/length(x)){
+  n<-length(x)
+  a<-interval[1]
+  b<-interval[2]
+  #cat("del=",del,"\n")
+  if(!is.null(z)){
+    #res <- ks::kde(x, h=h2mH(x), eval.points = z)
+    res <- ks::kde(x, eval.points = z)
+    fz<-res$estimate
+    h<-res$h
+  }
+  else {
+    if(nz==0) stop("Number of modes 'nz' is missing.")
+    modes<-multimode::locmodes(x, mod0=nz, lowsup=a, uppsup=b) 
+    z<-modes$locations[2*(1:nz)-1]
+    #fz0<-modes$fvalue[2*(1:nz)-1]
+    fz<-modes$fvalue[2*(1:nz)-1]
+    h<-modes$cbw$bw
+    #fz<-fz0
+  }
+  I0<-(z==a | z==b)
+  I1<-(z>a & z<b)
+  ufz<-vfz<-z
+  if(sum(I1)>0){
+    K2 <- .5 /sqrt(pi)
+    #res <- ks::kde(x, h=h2mH(x), eval.points = z[I1])
+    #fz[I1]<-res$estimate
+    #h<-res$h
+    # Estimated variance
+    vfz[I1] <- fz[I1]*K2/(n*h)
+  }
+  if(sum(I0)>0){
+    #cat("z=",z,"\n")
+    nz0<-sum(I0)
+    for(i in 1:nz0){
+      fz[I0][i]<-mean(abs(x-z[I0][i])<=del)/del
+      vfz[I0][i]<-fz[I0][i]/(n*del)
+    }
+  }
+  # upper limit of CI 
+  zalpha <- qnorm(1-alpha)
+  ufz <- fz + zalpha*sqrt(vfz)
+  ans<-list(z=z, ufz=ufz, fz=fz)
+  #if(is.null(z)) ans$fz0=fz0
+  ans
+}
+
+#####################################
+#' Method of mode estimate of a Bernstein polynomial model degree
+#' @description Select a Bernstein polynomial model degree for density  
+#'    on [0,1] based on mode(s) used as an initial guess for the iterative method
+#'    of moment of choosing the model degree
+#' @param modes a list containing \code{z}, a vector of the known or estimated 
+#'       locations of modes,  \code{ufz}, a vector of upper confidence limits
+#'       of density values at modes, and \code{fz}, a vector of estimated density 
+#'       values at modes by \code{ks::kde} or \code{multimode::locmodes()} 
+#' @param x a vector sample values 
+#' @param interval support/truncation interval
+#' @return  A list with components
+#' \itemize{
+#'   \item \code{mode} a vector of modes
+#'   \item \code{nmod} the number of modes in \code{mode}
+#'   \item \code{m}  the method of mode estimate of degree 
+#'   \item \code{lam} a vector of rough estimates of the mixing proportions 
+#'  }
+#' @author Zhong Guan <zguan@iusb.edu>
+#' @keywords internal
+ 
+momodem<-function(modes, x){
+  z<-modes$z
+  nz<-length(z)
+  fz<-modes$ufz
+  s<-sum(sqrt(z*(1-z))*fz*(z>0 & z<1))
+  s<-c(sum(fz*(z==0 | z==1)),s)
+  m<-(sqrt(pi)*s[2]+sqrt(pi*s[2]^2-2*(1-s[1])))^2/2
+  #cat("initialm: m=",m,"s=",s,"\n")
+  #cat("initialm: mz=",m*z,"\n")
+  while(any(m*z[z>0]<1 | m*(1-z[z<1])<1)){
+    if(any(m*z[z>0]<1)) {
+      z[m*z[z>0]<1]<-0
+      fz[m*z[z>0]<1]<-ucl.fz(x, z=0)$ufz
+    }
+    if(any(m*(1-z[z<1])<1)) {
+      z[m*(1-z[z<1])<1]<-1
+      fz[m*(1-z[z<1])<1]<-ucl.fz(x, z=1)$ufz
+    }   
+    s<-sum(sqrt(z*(1-z))*fz*(z>0 & z<1))
+    s<-c(sum(fz*(z==0 | z==1)),s)
+    m<-(sqrt(pi)*s[2]+sqrt(pi*s[2]^2-2*(1-s[1])))^2/2
+  }
+  #cat("initialm: m=",m,"s=",s,"\n")
+  lam<-sqrt(z*(1-z))*fz*sqrt(2*pi*m)/(m+1)
+  lam[z==0 | z==1]<-fz/(m+1)
+  list(mode=z, nmod=nz, m=ceiling(m), lam=lam)
+}
+#######################################################################
+#' mable with degree selected by the method of moment and method of mode
+#' @description Maximum Approximate Bernstein/Beta Likelihood Estimation
+#'      with an optimal model degree estimated by the Method of Moment
+#' @param x a univariate sample data in \code{interval}
+#' @param interval a closed interval \code{c(a,b)}, default is [0,1]
+#' @param m initial degree, default is 2 times the number of modes \code{nmod}. 
+#' @param mu a vector of component means of multimodal mixture density, 
+#'           default is NULL for unimodal or unknown
+#' @param lam a vector of mixture proportions of same length of \code{mu}
+#' @param modes a vector of the locations of modes, if it is NULL (default) and 
+#           \code{nmod} >1 then the modes will be estimated by 
+#'          \code{multimode::locmodes()} 
+#' @param nmod the number of modes, if \code{nmod}=0, the lower bound for
+#'          m is estimated based on mean and variance only.
+#' @param ushaped logical, whether or not the density is clearly U-shaped 
+#'          including J- and L-shaped with mode occurs at the endpoint of the support.
+#' @param maxit maximum iterations
+#' @details If the data show a clear uni- or multi-modal distribution, then give 
+#'          the value of \code{nmod} as the number of modes. Otherwise \code{nmod}=0.
+#'          The degree is estimated by the iterative method of moment with an initial
+#'          degree estimated by the method of mode. For multimodal density, 
+#'          if useful estimates of the component means \code{mu} and proportions 
+#'          \code{lam} are available then they can be used to give an initial degree.  
+#'          If the distribution is clearly U-, J-, or L-shaped, i.e., the mode occurs
+#'          at the endpoint of \code{interval}, then set \code{ushaped}=TRUE. 
+#'          In this case the degree is estimated by the method of mode. 
+#'           
+#'   
+#' @return  A class "mable" object with components
+#' \itemize{
+#'   \item \code{m} the given or a selected degree by method of change-point
+#'   \item \code{p} the estimated vector of mixture proportions \eqn{p = (p_0, \ldots, p_m)}
+#'       with the selected/given optimal degree \code{m}
+#'   \item \code{mloglik}  the maximum log-likelihood at degree \code{m}
+#'   \item \code{interval} support/truncation interval \code{(a,b)}
+#'   \item \code{convergence} An integer code. 0 indicates successful completion 
+#'      (all the EM iterations are convergent and an optimal degree
+#'      is successfully selected in \code{M}). Possible error codes are
+#'    \itemize{
+#'       \item 1, indicates that the iteration limit \code{maxit} had been 
+#'          reached in at least one EM iteration;
+#'       \item 2, the search did not finish before \code{m1}.
+#'     }
+#'   \item \code{delta} the convergence criterion \code{delta} value
+#'  }
+#' @examples
+#' \donttest{
+#' ## Old Faithful Data
+#' x<-faithful
+#' x1<-faithful[,1]
+#' x2<-faithful[,2]
+#' a<-c(0, 40); b<-c(7, 110)
+#' mu<-(apply(x,2,mean)-a)/(b-a)
+#' s2<-apply(x,2,var)/(b-a)^2
+#' # mixing proportions
+#' lambda<-c(mean(x1<3),mean(x2<65))
+#' # guess component mean
+#' mu1<-(c(mean(x1[x1<3]), mean(x2[x2<65]))-a)/(b-a)  
+#' mu2<-(c(mean(x1[x1>=3]), mean(x2[x2>=65]))-a)/(b-a)  
+#' # estimate lower bound for m
+#' mb<-ceiling((mu*(1-mu)-s2)/(s2-lambda*(1-lambda)*(mu1-mu2)^2)-2)
+#' mb
+#' m1<-optimable(x1, interval=c(a[1],b[1]), nmod=2, modes=c(2,4.5))$m 
+#' m2<-optimable(x2, interval=c(a[2],b[2]), nmod=2, modes=c(52.5,80))$m 
+#' m1;m2
+#' erupt1<-mable(x1, M=mb[1], interval=c(a[1],b[1]))
+#' erupt2<-mable(x1, M=m1, interval=c(a[1],b[1])) 
+#' wait1<-mable(x2, M=mb[2],interval=c(a[2],b[2])) 
+#' wait2<-mable(x2, M=m2,interval=c(a[2],b[2])) 
+#' ans1<- mable.mvar(faithful, M = mb, search =FALSE, interval = cbind(a,b))
+#' ans2<- mable.mvar(faithful, M = c(m1,m2), search =FALSE, interval = cbind(a,b))
+#' op<-par(mfrow=c(1,2), cex=0.8)
+#' hist(x1, probability = TRUE, col="grey", border="white", main="", 
+#'       xlab="Eruptions", ylim=c(0,.65), las=1)
+#' plot(erupt1, add=TRUE,"density")
+#' plot(erupt2, add=TRUE,"density",lty=2,col=2)
+#' legend("topleft", lty=c(1,2),col=1:2, bty="n", cex=.7,  
+#'       c(expression(paste("m = ", m[b])),expression(paste("m = ", hat(m)))))
+#' hist(x2, probability = TRUE, col="grey", border="white", main="", 
+#'       xlab="Waiting", las=1)
+#' plot(wait1, add=TRUE,"density")
+#' plot(wait2, add=TRUE,"density",lty=2,col=2)
+#' legend("topleft", lty=c(1,2),col=1:2, bty="n", cex=.7,  
+#'       c(expression(paste("m = ", m[b])),expression(paste("m = ", hat(m)))))
+#' par(op)
+#' op<-par(mfrow=c(1,2), cex=0.7)
+#' plot(ans1, which="density", contour=TRUE) 
+#' plot(ans2, which="density", contour=TRUE, add=TRUE, lty=2, col=2) 
+#' plot(ans1, which="cumulative", contour=TRUE)       
+#' plot(ans2, which="cumulative", contour=TRUE, add=TRUE, lty=2, col=2)
+#' par(op)
+#' }
+#' @author Zhong Guan <zguan@iusb.edu>
+#' @export
+
+
+optimable<-function(x, interval, m=NULL, mu=NULL, lam=NULL, modes=NULL, nmod=1, 
+             ushaped=FALSE,  maxit=50){
+  M<-NULL
+  if(missing(interval)) interval=c(0,1)
+  n<-length(x)
+  y<-(x-interval[1])/diff(interval);
+  Nut<-c(0,0)
+  nu<-mean(y)
+  a<-var(y)  
+  b<-nu*(1-nu)-a;
+  if(!is.null(mu)){
+    nmu<-length(mu)
+    mu<-(mu-interval[1])/diff(interval)
+    if(nmu==1){
+      message("'mu' is of length 1 and ignored." )}
+    else if(nmu!=length(lam) || any(lam<0)) 
+      stop("'mu' and 'lam' have different lengths or negative 'lam'.")
+    lam<-lam/sum(lam)
+    mb<-max(0,ceiling(b/(a-sum(lam*(mu-nu)^2))-2))
+    m0<-ifelse(is.null(m), mb,m)
+    #cat("mb=", mb, "\n",sep='')
+  }
+  else{
+    if(nmod>0){  
+      if(!is.null(modes)) z<-(modes-interval[1])/diff(interval)
+      modes<-ucl.fz(y, z=z, nz=nmod)
+      obj<-momodem(modes,y) 
+      m0<-obj$m
+      z<-obj$mode
+      #lam<-obj$lam
+      if(nmod>1) mb<-max(m0,ifelse(is.null(m), 2*(nmod-sum(z==0 | z==1)), m))
+      else mb<-max(m0,round(b/a-2))
+      #cat("mu=", mu, ", lam=", lam, "\n", sep=' ')
+      #cat("nmode=", nmod, ", mb=", mb, ", m0=", m0, "\n",sep='')
+    }
+    else{
+      mb<-max(0,round(b/a-2))
+      #cat("nmode=", nmod,", mb=", mb, "\n",sep='')
+      m0<-ifelse(is.null(m), mb, m)
+    }
+  }
+  #m0<-ifelse(is.null(m), max(0,round(b/a-2)),m)
+  #cat("mb=", mb, ", m0=", m0, "\n",sep='')
+  M[1]<-mb
+  m<-M[1]
+  it=0
+  cat("it=", it, ",  M[",it,"]=",m, "\n",sep='')
+  res<-mable(y, M=m, interval=c(0,1))
+  if(!ushaped){
+    p<-res$p
+    Pij<-(1:(m+1))/(m+2)
+    Nut[1]<-sum(Pij*p)
+    Nut[2]<-sum(Pij^2*p)
+    c<-Nut[2]-Nut[1]^2
+    m<-round(max(0,b/(a-c)-2))
+    Stop=FALSE
+    while(it<maxit){ 
+      it<-it+1
+      res<-mable(y, M=m, interval=c(0,1))
+      p<-res$p
+      Pij<-(1:(m+1))/(m+2)
+      Nut[1]<-sum(Pij*p)
+      Nut[2]<-sum(Pij^2*p)
+      c<-Nut[2]-Nut[1]^2
+      while(any(is.na(p)) || a<c) {
+        m<-m-1
+        res<-mable(y, M=m, interval=c(0,1))
+        p<-res$p
+        Pij<-(1:(m+1))/(m+2)
+        Nut[1]<-sum(Pij*p)
+        Nut[2]<-sum(Pij^2*p)
+        c<-Nut[2]-Nut[1]^2
+      }
+      m<-round(max(0,b/(a-c)-2))
+      M[it]<-m
+      for(j in (it-1):1){
+        test<-rep.check(M[j:it])
+        reps<-test$Reps
+        if(reps>=2) {
+          m<-round(mean(test$Subseq))
+          cat("it=", it, ",  M[",it,"]=",m, "\n",sep='')
+          #cat("it=", it, ", reps=", reps, "\n",sep='')
+          Stop=TRUE
+          break}
+      }
+      if(Stop) break
+      cat("it=", it, ",  M[",it,"]=",m, "\n",sep='')
+    }
+    if(it==maxit) message("Maximum iteration reached.\n")
+    #m<-max(mb,m)
+    M[it]<-m
+    res<-mable(x, M=m, interval)
+  }
+  else res$interval<-interval
+  cat("Number of iterations = ", it, ", m=",m, "\n",sep='')
+  res$mb<-mb
+  res$M=M
+  res$N=it
+  #res$m=m
+  res
+}
+
+
 ##############################################
 #' Mixture Beta Distribution
 #' @description Density, distribution function, quantile function and
@@ -471,10 +807,11 @@ dmixbeta<-function(x, p, interval=c(0, 1)){
     n<-length(x)
     a<-interval[1]
     b<-interval[2]
+    eps<-.Machine$double.eps^.25
     if(a>=b) stop("a must be smaller than b")
-    if(any(x<a) || any(x>b)) stop("Argument 'x' must be in 'interval'.")
+    #if(any(x<a) || any(x>b)) stop("Argument 'x' must be in 'interval'.")
     if(length(p)==0) stop("Missing mixture proportions 'p' without default.")
-    if(any(p<0) || abs(sum(p)-1)>.Machine$double.eps^.25) 
+    if(any(p+eps^2<0) || abs(sum(p)-1)>eps) 
       message("Components of 'p' are not all nonnegative or do not sum to 1.")
     m<-length(p)-1
     if(m==0) y<-dunif(x, a, b)
@@ -493,10 +830,11 @@ pmixbeta<-function(x, p, interval=c(0, 1)){
     n<-length(x)
     a<-interval[1]
     b<-interval[2]
+    eps<-.Machine$double.eps^.25
     if(a>=b) stop("a must be smaller than b")
-    if(any(x<a) || any(x>b)) stop("Argument 'x' must be in 'interval'.")
+    #if(any(x<a) || any(x>b)) stop("Argument 'x' must be in 'interval'.")
     if(length(p)==0) stop("Missing mixture proportions 'p' without default.")
-    if(any(p<0) || abs(sum(p)-1)>.Machine$double.eps^.25) 
+    if(any(p+eps^2<0) || abs(sum(p)-1)>eps) 
       message("Components of 'p' are not all nonnegative or do not sum to 1.")
     m<-length(p)-1
     if(m==0) y<-punif(x, a, b)
@@ -517,10 +855,11 @@ pmixbeta<-function(x, p, interval=c(0, 1)){
 qmixbeta<-function(u, p, interval=c(0, 1)){
     a<-interval[1]
     b<-interval[2]
+    eps<-.Machine$double.eps^.25
     if(a>=b) stop("a must be smaller than b")
     if(any(u<0) || any(u>1)) stop("Argument 'u' must be in [0,1].")
     if(length(p)==0) stop("Missing mixture proportions 'p' without default.")
-    if(any(p<0) || abs(sum(p)-1)>.Machine$double.eps^.25) 
+    if(any(p+eps^2<0) || abs(sum(p)-1)>eps)
       message("Components of 'p' are not all nonnegative or do not sum to 1.")
     m<-length(p)-1
     if(m==0) Q<-qunif(u, a, b)
@@ -543,10 +882,11 @@ qmixbeta<-function(u, p, interval=c(0, 1)){
 rmixbeta<-function(n, p, interval=c(0, 1)){
     a<-interval[1]
     b<-interval[2]
+    eps<-.Machine$double.eps^.25
     if(a>=b) stop("a must be smaller than b")
     if(length(p)==0) stop("Missing mixture proportions 'p' without default.")
     if(any(p<0)) stop("Negative component(s) of argument 'p'is not allowed.")
-    if(abs(sum(p)-1)>.Machine$double.eps^.25){
+    if(any(p+eps^2<0) || abs(sum(p)-1)>eps){
         message("Sum of 'p's is not 1. Dividing 'p's by the total.")
         p<-p/sum(p)
     }
@@ -571,6 +911,7 @@ rmixbeta<-function(n, p, interval=c(0, 1)){
 #'  "density", "cumulative", "likelihood", "change-point", "all". If not "all",
 #'  \code{which} can contain more than one options.
 #' @param add logical add to an existing plot or not
+#' @param contour logical plot contour or not for two-dimensional data
 #' @param lgd.x,lgd.y coordinates of position where the legend is displayed
 #' @param nx  number of evaluations of density, or cumulative distribution curve to be plotted.
 #' @param ...  additional arguments to be passed to the base plot function
@@ -578,103 +919,107 @@ rmixbeta<-function(n, p, interval=c(0, 1)){
 #' @return The data used for 'plot()', 'lines()', or 'persp()' are returned invisibly.
 #' @export
 plot.mable<-function(x, which=c("density", "cumulative", "survival", "likelihood",
-                "change-point", "all"), add = FALSE,
-                lgd.x=NULL, lgd.y=NULL, nx=512,...){
-    phat<-x$p
-    m<-x$m
-    if(is.null(x$dim)) dim<-1
-    else dim<-x$dim
-    if(dim>2) stop("There is no method to 'plot' the object.\n")
-    support<-x$interval
-    if(is.null(lgd.x)) lgd.x="topright"
-    which <- match.arg(which, several.ok=TRUE)
-    if(dim==1){
-        a<-support[1]
-        b<-support[2]
-        xx<-seq(a, b, len=nx)
-        if(any(which=="all")){
-            add=FALSE
-            op<-par(mfrow=c(2,2))}
+              "change-point", "all"), add = FALSE, contour = FALSE,
+              lgd.x=NULL, lgd.y=NULL, nx=512,...){
+  obj<-x
+  phat<-obj$p
+  m<-obj$m
+  dim<-length(m)
+  if(dim>2) stop("There is no method to 'plot' the object.\n")
+  support<-obj$interval
+  if(is.null(lgd.x)) lgd.x="topright"
+  which <- match.arg(which, several.ok=TRUE)
+  if(dim==1){
+    a<-support[1]
+    b<-support[2]
+    x<-seq(a, b, len=nx)
+    if(any(which=="all")){
+      add=FALSE
+      op<-par(mfrow=c(2,2))}
     if(any(which=="likelihood")|| any(which=="all")){
-        if(is.null(x$lk)) message("Cannot plot 'likelihood'.")
-        M<-x$M[1]:x$M[2]
-        if(!add) plot(M, x$lk, type="p", xlab="m",ylab="Loglikelihood",
-            main="Loglikelihood")
-        else points(M, x$lk, pch=1)
-        segments(x$m, min(x$lk), x$m, x$mloglik, lty=2, col=1:2)
-        axis(1, x$m,  as.character(x$m))
-        out<-list(x=M, y=x$lk)
+      if(is.null(obj$lk)) message("Cannot plot 'likelihood'.")
+      M<-obj$M[1]:obj$M[2]
+      if(!add) plot(M, obj$lk, type="p", xlab="m",ylab="Loglikelihood",
+          main="Loglikelihood")
+      else points(M, obj$lk, pch=1)
+      segments(obj$m, min(obj$lk), obj$m, obj$mloglik, lty=2, col=1:2)
+      axis(1, obj$m,  as.character(obj$m))
+      out<-list(x=M, y=obj$lk)
     }
     if(any(which=="change-point")||any(which=="all")) {
-        if(is.null(x$lr)) message("Cannot plot 'likelihood ratios'.")
-        M<-x$M[1]:x$M[2]
-        ymin<-min(x$lr)
-        ymax<-max(x$lr)
-        if(!add){
-            plot(M, 0*M, type="n", xlab="m", ylab="", ylim=c(ymin,ymax))
-            points(M[-1], x$lr, col=1, pch=1, ylab="Likelihood Ratio")
-            segments(m, ymin, m, ymax, lty=2, col=1)
-            axis(1, m,  as.character(m), col=1)
-            title("LR of Change-Point")}
-        else{
-            points(M[-1], x$lr, ...)
-            segments(m, ymin, m, max(x$lr), ...)
-            axis(1, m,  as.character(m), ...)}
-        out<-list(x=M[-1], y=x$lr)
+      if(is.null(obj$lr)) message("Cannot plot 'likelihood ratios'.")
+      M<-obj$M[1]:obj$M[2]
+      ymin<-min(obj$lr)
+      ymax<-max(obj$lr)
+      if(!add){
+        plot(M, 0*M, type="n", xlab="m", ylab="", ylim=c(ymin,ymax))
+        points(M[-1], obj$lr, col=1, pch=1, ylab="Likelihood Ratio")
+        segments(m, ymin, m, ymax, lty=2, col=1)
+        axis(1, m,  as.character(m), col=1)
+        title("LR of Change-Point")}
+      else{
+        points(M[-1], obj$lr, ...)
+        segments(m, ymin, m, max(obj$lr), ...)
+        axis(1, m,  as.character(m), ...)}
+      out<-list(x=M[-1], y=obj$lr)
     }
     if(any(which=="density")||any(which=="all")){
-        yy<-dmixbeta(xx, phat[1:(m+1)], c(a, b))
-        if(!add)
-            plot(xx, yy, type="l",  ylab="Density", xlim=c(a, b), ...)
-        else
-            lines(xx, yy, ...)
-        out<-list(x=xx, y=yy)
+      yy<-dmixbeta(x, phat[1:(m+1)], c(a, b))
+      if(!add)
+          plot(x, yy, type="l",  ylab="Density", xlim=c(a, b), ...)
+      else
+          lines(x, yy, ...)
+      out<-list(x=x, y=yy)
     }
     if(any(which=="cumulative")||any(which=="survival")||any(which=="all")){
-        if(any(which=="survival")||any(which=="all"))
-            yy<-1-pmixbeta(xx, phat[1:(m+1)], c(a, b))
-        else
-            yy<-pmixbeta(xx, phat[1:(m+1)], c(a, b))
-        if(!add) {
-            if(any(which=="survival")||any(which=="all")) plot(xx, yy,
-                type="l", ylab="Survival Probability", xlim=c(a,b), col=1, ...)
-            else plot(xx, yy, type="l", ylab="Cumulative Probability", xlim=c(a,b), col=1, ...)}
-        else lines(xx, yy, ...)
-        out<-list(x=xx, y=yy)}
-        if(any(which=="all")) par(op)
-        #out<-list(x=xx, y=yy)
+      if(any(which=="survival")||any(which=="all"))
+        yy<-1-pmixbeta(x, phat[1:(m+1)], c(a, b))
+      else
+        yy<-pmixbeta(x, phat[1:(m+1)], c(a, b))
+      if(!add) {
+        if(any(which=="survival")||any(which=="all")) plot(x, yy,
+          type="l", ylab="Survival Probability", xlim=c(a,b), col=1, ...)
+        else plot(x, yy, type="l", ylab="Cumulative Probability", xlim=c(a,b), col=1, ...)}
+      else lines(x, yy, ...)
+      out<-list(x=x, y=yy)}
+      if(any(which=="all")) par(op)
+      #out<-list(x=x, y=yy)
+  }
+  else{
+    a<-support[,1]
+    b<-support[,2]
+    if(!any(which=="cumulative")&& !any(which=="density")&& !any(which=="all")){
+      cat("Only 'density' and 'cumulative' distribution can be plotted.\n")
+      which<-"all"
     }
-    else{
-        a<-support[,1]
-        b<-support[,2]
-        if(!any(which=="cumulative")&& !any(which=="density")&& !any(which=="all")){
-            cat("Only 'density' and 'cumulative' distribution can be plotted.\n")
-            which<-"all"
-        }
-        if(any(which=="all")) op<-par(mfrow=c(2,1))
-        if(any(which=="density")||any(which=="all")){
-            fn<-function(x1, x2) dmixmvbeta(cbind(x1, x2), phat, m, interval=support)
-            x1 <- seq(a[1], b[1], length= 40)
-            x2 <- seq(a[2], b[2], length= 40)
-             z <- outer(x1, x2, fn)
-            persp(x1, x2, z, theta = 30, phi = 20, expand = 0.5, col = "lightblue",
-              ltheta = 90, shade = 0.1, ticktype = "detailed", main = expression(paste("MABLE ",hat(f))),
-              xlab = x$xNames[1], ylab = x$xNames[2], zlab = "Joint Density")
-            out<-list(x1=x1,x2=x2,z=z)
-        }
-        if(any(which=="cumulative")||any(which=="all")){
-            fn<-function(x1, x2) pmixmvbeta(cbind(x1, x2), phat, m, interval=support)
-            x1 <- seq(a[1], b[1], length= 40)
-            x2 <- seq(a[2], b[2], length= 40)
-            z  <- outer(x1, x2, fn)
-            persp(x1, x2, z, theta = 30, phi = 20, expand = 0.5, col = "lightblue",
-              ltheta = 90, shade = 0.1, ticktype = "detailed", main = expression(paste("MABLE ",hat(F))),
-              xlab = x$xNames[1], ylab = x$xNames[2], zlab = "Joint CDF")
-            out<-list(x1=x1,x2=x2,z=z)
-        }
-        if(any(which=="all")) par(op)
+    if(any(which=="all")) op<-par(mfrow=c(2,1))
+    if(any(which=="density")||any(which=="all")){
+      fn<-function(x1, x2) dmixmvbeta(cbind(x1, x2), phat, m, interval=support)
+      x1 <- seq(a[1], b[1], length= 40)
+      x2 <- seq(a[2], b[2], length= 40)
+       z <- outer(x1, x2, fn)
+      if(contour) contour(x1, x2, z, add=add, main = expression(paste("MABLE ",hat(f))),
+        xlab = obj$xNames[1], ylab = obj$xNames[2],...)
+      else persp(x1, x2, z, theta = 30, phi = 20, expand = 0.5, col = "lightblue",
+        ltheta = 90, shade = 0.1, ticktype = "detailed", main = expression(paste("MABLE ",hat(f))),
+        xlab = obj$xNames[1], ylab = obj$xNames[2], zlab = "Joint Density")
+      out<-list(x1=x1,x2=x2,z=z)
     }
-    invisible(out)
+    if(any(which=="cumulative")||any(which=="all")){
+      fn<-function(x1, x2) pmixmvbeta(cbind(x1, x2), phat, m, interval=support)
+      x1 <- seq(a[1], b[1], length= 40)
+      x2 <- seq(a[2], b[2], length= 40)
+      z  <- outer(x1, x2, fn)
+      if(contour) contour(x1, x2, z, add=add, main = expression(paste("MABLE ",hat(F))),
+        xlab = obj$xNames[1], ylab = obj$xNames[2],...)
+      else persp(x1, x2, z, theta = 30, phi = 20, expand = 0.5, col = "lightblue",
+        ltheta = 90, shade = 0.1, ticktype = "detailed", main = expression(paste("MABLE ",hat(F))),
+        xlab = obj$xNames[1], ylab = obj$xNames[2], zlab = "Joint CDF")
+      out<-list(x1=x1,x2=x2,z=z)
+    }
+    if(any(which=="all")) par(op)
+  }
+  invisible(out)
 }
 ##############################################
 #' Summary mathods for classes 'mable' and 'mable_reg'
@@ -710,8 +1055,7 @@ summary.mable<-function(object, ...){
     else ans$m<-obj$m
     ans$mloglik<-obj$mloglik
     ans$interval<-obj$interval
-    if(is.null(obj$dim)) ans$dim<-1
-    else  ans$dim<-obj$dim
+    ans$dim<-length(obj$m)
     ans$p<-obj$p
     ans$pval<-obj$pval[length(obj$pval)]
     switch(obj$data.type,
@@ -753,6 +1097,7 @@ summary.mable_reg<-function(object, ...){
     ans$dimcov<-length(obj$x0)
     ans$se<-obj$SE
     ans$z<-obj$z
+    ans$pval.z<-2*pnorm(-abs(ans$z))
     ans$p<-obj$p
     ans$pval<-obj$pval[length(obj$pval)]
     cat(paste("Call: ",obj$model,"(",obj$callText,")", sep=''))
@@ -765,8 +1110,8 @@ summary.mable_reg<-function(object, ...){
     cat("Maximum loglikelihood:", ans$mloglik,"\n")
     cat("MABLE of p: can be retrieved using name 'p' \n")
     cat("\n")
-    prnt<-cbind(ans$coefficients, ans$se, ans$z, ans$baseline)
-    colnames(prnt)<-c("Estimate", "Std.Err", "Z value", "Baseline x0")
+    prnt<-cbind(ans$coefficients, ans$se, ans$z, ans$pval.z, ans$baseline)
+    colnames(prnt)<-c("Estimate", "Std.Error", "z value", "Pr(>|z|)", "Baseline x0")
     rownames(prnt)<-ans$names
     printCoefmat(prnt)
     cat("\nNote: the optimal model degree is selected by the method of change-point.\n")
